@@ -70,7 +70,7 @@ static struct
 } filecap_state;
 
 static capstatus_t capture_status = STOP;
-static gint capture_source = -1;
+static guint capture_source = 0;           /* ID of timeout/channel source */
 
 static inline gchar *appdata_source_name(const struct appdata_struct *p)
 {
@@ -250,6 +250,7 @@ GList *get_capture_interfaces(GString *err)
 
   for (devname = raw_msg; *devname; devname += strlen(devname) + 1)
     ifs = g_list_prepend(ifs, g_strdup(devname));
+  ifs = g_list_reverse(ifs);  /* restore original order */
 
   g_free(raw_msg);
   g_string_assign(err, "");
@@ -291,12 +292,15 @@ static gchar *start_live_capture(unsigned int *linktype, int *select_fd)
   struct capctl_resp_t resp;
   gchar *new_devname;
 
+  if (!appdata.source.interface || !strlen(appdata.source.interface))
+    return g_strdup(_("no capture interface specified, capture aborted"));
+
   zeroreq(&req);
 
   g_assert(appdata.source.type == ST_LIVE);
 
   req.type = CRQ_STARTCAP;
-  req.startcap.devlen = appdata.source.interface ? strlen(appdata.source.interface) : 0;
+  req.startcap.devlen = strlen(appdata.source.interface);
 
   sendreq(&req);
   if (appdata.source.interface)
@@ -402,11 +406,7 @@ static gboolean filecap_get_packet(gpointer unused)
 
 static void filecap_timeout_destroy(gpointer unused)
 {
-  if (capture_status == STOP) {
-    pcap_close(filecap_state.pcap);
-    filecap_state.pcap = NULL;
-  }
-  else if (capture_status == PLAY)
+  if (capture_status == PLAY)
     capture_source = g_timeout_add_full(G_PRIORITY_DEFAULT, filecap_state.wait_ms,
                                         filecap_get_packet, NULL, filecap_timeout_destroy);
 }
@@ -430,54 +430,45 @@ static gchar *start_file_capture(unsigned int *linktype)
   return NULL;
 }
 
-gchar *start_capture(void)
+// real capture initialization
+static gchar *start_capture_internal(void)
 {
-  gchar *err;
+  gchar *err = NULL;
   unsigned int linktype;
   int select_fd = -1;
 
-  if (capture_status == STOP) {
-    protocol_summary_open();
-    nodes_catalog_open();
-    links_catalog_open();
-  }
-
   if (appdata.source.type == ST_LIVE)
     err = start_live_capture(&linktype, &select_fd);
   else
     err = start_file_capture(&linktype);
 
-  if (err) {
-    links_catalog_close();
-    nodes_catalog_close();
-    protocol_summary_close();
+  if (err)
     return err;
-  }
 
-  /* FIXME: stop capture on errors past here */
+  // from now, capture is marked active, even if it's not fully initialized
+  capture_status = PLAY;
 
-  if (!setup_link_type(linktype))
+  if (!setup_link_type(linktype)) {
     return g_strdup_printf(_("%s uses unsupported link type %d, cannot proceed."
                              "  Please choose another source."),
                            appdata_source_name(&appdata), linktype);
+  }
 
   if (appdata.mode == APEMODE_DEFAULT) {
     appdata.mode = IP;
     g_free(pref.filter);
     pref.filter = get_default_filter(appdata.mode);
   }
-  else if (appdata.mode == LINK6 && !has_linklevel())
+  else if (appdata.mode == LINK6 && !has_linklevel()) {
     return g_strdup_printf(_("This device does not support link-layer mode.  "
                              "Please use IP or TCP modes."));
-
+  }
   if (pref.filter) {
     err = set_filter(pref.filter);
     if (err)
       return err;
   }
 
-  capture_status = PLAY;
-
   if (appdata.source.type == ST_LIVE) {
     GIOChannel *channel = g_io_channel_unix_new(select_fd);
     capture_source = g_io_add_watch(channel,
@@ -494,6 +485,25 @@ gchar *start_capture(void)
   return NULL;
 }
 
+// capture start entry point
+gchar *start_capture(void)
+{
+  if (capture_status == STOP) {
+    protocol_summary_open();
+    nodes_catalog_open();
+    links_catalog_open();
+  }
+
+  gchar *errstr = start_capture_internal();
+  if (errstr) {
+    stop_capture();
+    return errstr;
+  }
+
+  // capture started
+  return NULL;  
+}
+
 gchar *pause_capture(void)
 {
   g_assert(capture_status == PLAY);
@@ -506,7 +516,7 @@ gchar *unpause_capture(void)
   g_assert(capture_status == PAUSE);
 
   if (appdata.source.type == ST_FILE)
-    capture_status = g_timeout_add_full(G_PRIORITY_DEFAULT, 1,
+    capture_source = g_timeout_add_full(G_PRIORITY_DEFAULT, 1,
                                         filecap_get_packet, NULL,
                                         filecap_timeout_destroy);
 
@@ -516,74 +526,60 @@ gchar *unpause_capture(void)
 
 static gchar *stop_live_capture(void)
 {
-  struct capctl_req_t req;
-  struct capctl_resp_t resp;
-
   g_assert(appdata.source.type == ST_LIVE);
 
-  zeroreq(&req);
+  if (capture_source > 0){
+    g_source_remove(capture_source);
+  capture_source = 0;
+}
 
-  g_source_remove(capture_source);
+  struct capctl_req_t req;
+  zeroreq(&req);
 
   req.type = CRQ_STOPCAP;
   sendreq(&req);
 
+  struct capctl_resp_t resp;
   recvresp(&resp);
 
   if (resp.status == CRP_ERR)
     return recvstr(resp.err.msglen);
 
   g_assert(resp.status == CRP_OK);
-
   return NULL;
 }
 
-gchar *stop_capture(void)
+void stop_capture(void)
 {
-  gchar *err = NULL;
+  // state is set to stop now, giving callbacks a chance to exit
   capstatus_t orig_state = capture_status;
-
-  g_assert(capture_status == PLAY || capture_status == PAUSE ||
-           capture_status == CAP_EOF);
-
-  /*
-   * HACK: setting capture_status = STOP here (before the actual stop
-   * operation) so that callbacks for file capture happening via timers will
-   * see it and tear down.  We save the original state and restore it on
-   * failure (kind of gross).
-   */
   capture_status = STOP;
 
+  gchar *err = NULL;
   if (appdata.source.type == ST_LIVE)
     err = stop_live_capture();
   else {
-    if (orig_state == PLAY) {
-      /*
-       * If we're playing, the timeout destroy function will take care of
-       * cleaning up closing and NULLing filecap_state.pcap.
-       */
+    if (orig_state == PLAY && capture_source > 0) {
       g_source_remove(capture_source);
+      capture_source = 0;
     }
-    else if (orig_state == PAUSE || orig_state == CAP_EOF) {
-      /*
-       * But if we're stopping from a pause the timeout's not armed and
-       * we'll need to do it here.
-       */
+
+    if (filecap_state.pcap) {
       pcap_close(filecap_state.pcap);
       filecap_state.pcap = NULL;
     }
   }
 
-  if (!err) {
-    links_catalog_close();
-    nodes_catalog_close();
-    protocol_summary_close();
-    new_nodes_clear();
-  }
-  else
-    capture_status = orig_state;
+  links_catalog_close();
+  nodes_catalog_close();
+  protocol_summary_close();
+  new_nodes_clear();
 
-  return err;
+  // error or not, capture is marked stopped
+  if (err) {
+    g_error("failed to stop capture: %s", err);
+    g_free(err);
+  }
 }
 
 static gchar *cleanup_live_capture(void)
@@ -627,24 +623,15 @@ void cleanup_capture(void)
 {
   gchar *err = NULL;
 
-  if (capture_status != STOP) {
-    err = stop_capture();
-    if (err) {
-      g_error("failed to stop capture: %s", err);
-      g_free(err);
-    }
-  }
-
-  if (appdata.source.type == ST_FILE)
-    g_assert(!filecap_state.pcap);
+  if (capture_status != STOP)
+    stop_capture();
 
   /*
-   * This happens unconditionally because even if we've been replaying from a
-   * capture file, the live-capture process is still sitting around in the
-   * background and needs to be stopped.
-   */
+    * This happens unconditionally because even if we've been replaying from a
+    * capture file, the live-capture process is still sitting around in the
+    * background and needs to be stopped.
+    */
   err = cleanup_live_capture();
-
   if (err) {
     g_error("Capture cleanup error: %s", err);
     g_free(err);
@@ -655,7 +642,10 @@ void force_next_packet(void)
 {
   g_assert(capture_status == PLAY);
   filecap_state.wait_ms = 0;
-  g_source_remove(capture_source);
+  if (capture_source > 0){
+    g_source_remove(capture_source);
+    capture_source = 0;
+  }
 }
 
 static gchar *set_live_filter(const gchar *filter)
@@ -721,17 +711,17 @@ gchar *get_default_filter(apemode_t mode)
 {
   switch (mode)
   {
-      case IP:
-        return g_strdup("ip or ip6");
+  case IP:
+    return g_strdup("ip or ip6");
 
-      case TCP:
-        return g_strdup("tcp");
+  case TCP:
+    return g_strdup("tcp");
 
-      default:
-        g_error("Invalid apemode %d", mode);
-      /* Fallthrough */
-      case LINK6:
-        return g_strdup("");
+  default:
+    g_error("Invalid apemode %d", mode);
+  /* Fallthrough */
+  case LINK6:
+    return g_strdup("");
   }
 }
 
@@ -770,7 +760,10 @@ static gchar *get_filecap_stats(struct pcap_stat *ps)
 
 gchar *get_capture_stats(struct pcap_stat *ps)
 {
-  return (appdata.source.type == ST_FILE ? get_filecap_stats : get_live_stats)(ps);
+  if (appdata.source.type == ST_FILE)
+    return get_filecap_stats(ps);
+  else 
+    return get_live_stats(ps);
 }
 
 capstatus_t get_capture_status(void)
