@@ -90,7 +90,7 @@ static inline void ctrl_send(const void *buf, size_t len)
 static inline void ctrl_recv(void *buf, size_t len)
 {
   if (read_all(ctrlsock, buf, len)) {
-    xcritical("read_all() failed on control socket");
+    xerror("read_all() failed on control socket");
     exit(1);
   }
 }
@@ -192,12 +192,8 @@ static void handle_startcap(const struct capctl_req_t *req)
   }
 
   if (!req->startcap.devlen) {
-    devname = pcap_lookupdev(errbuf);
-    if (!devname) {
-      senderrmsg(errbuf);
-      return;
-    }
-    devname = xstrdup(devname);
+    senderrmsg("zero device length");
+    return;
   }
   else
     devname = recvstr(req->startcap.devlen);
@@ -292,10 +288,9 @@ static void handle_stopcap(const struct capctl_req_t *req)
     pcap_close(pcap);
     pcap = NULL;
     pcap_fd = -1;
-    sendok();
   }
-  else
-    senderrmsg("already stopped");
+
+  sendok();
 }
 
 static void handle_exit(const struct capctl_req_t *req)
@@ -382,7 +377,7 @@ static void handle_packet(void)
     assert(pktbuf.buffered <= sizeof(pktbuf.buf));
 
     memcpy(pktbuf.buf, hdr, sizeof(*hdr));
-    memcpy(pktbuf.buf + sizeof(*hdr), data, hdr->caplen);
+    memcpy(pktbuf.buf + sizeof(*hdr), data, hdr->caplen<MAXCAPSIZE ? hdr->caplen : MAXCAPSIZE);
 
     flush_pktbuf();
   }
