@@ -134,13 +134,13 @@ static inline int address_cmp(const address_t *a, const address_t *b)
     return memcmp(a->addr8, b->addr8, address_len(a->type));
 }
 
-#define g_my_debug(format, args ...)     g_log(G_LOG_DOMAIN, \
+#define g_my_debug(format, args ...)     g_log("etherape", \
                                                G_LOG_LEVEL_DEBUG, \
                                                format, ## args)
-#define g_my_info(format, args ...)      g_log(G_LOG_DOMAIN, \
+#define g_my_info(format, args ...)      g_log("etherape", \
                                                G_LOG_LEVEL_INFO, \
                                                format, ## args)
-#define g_my_critical(format, args ...)  g_log(G_LOG_DOMAIN, \
+#define g_my_critical(format, args ...)  g_log("etherape", \
                                                G_LOG_LEVEL_CRITICAL, \
                                                format, ## args)
 
