@@ -19,7 +19,7 @@
 #define DATASTRUCT_H
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 #include "stats/prot_types.h"
 
