@@ -18,11 +18,11 @@
  */
 
 #ifndef __EXPORT_H__
-#define __EXPORT__
+#define __EXPORT_H__
 
 #include "appdata.h"
 
-gchar *generate_xml(void);
-void dump_xml(gchar *ofile);
+gchar *generate_xml(unsigned long totpkts);
+void dump_xml(gchar *ofile, unsigned long totpkts);
 
 #endif                          /* __EXPORT_H__ */
