@@ -102,24 +102,27 @@ static GtkBuilder *new_bldr_instance(const gchar *id)
   GtkBuilder *bldr;
   gchar *ids[2];
 
-  ids[0] = id;
+  ids[0] = g_strdup(id);  /* Create a non-const copy */
   ids[1] = NULL;
 
   bldr = gtk_builder_new();
   if (!bldr) {
     g_error(_("We could not load the interface! (%s)"),
             appdata.itf_file);
+    g_free(ids[0]);
     return NULL;
   }
 
   if (!gtk_builder_add_objects_from_file(bldr, appdata.itf_file, ids, NULL)) {
+    g_free(ids[0]);
     g_error(_("Cannot create widget %s from file %s!"),
             id,
             appdata.itf_file);
     g_object_unref(bldr);
     return NULL;
   }
-
+  
+  g_free(ids[0]);  /* Free the duplicated string */
   gtk_builder_connect_signals(bldr, NULL);
   return bldr;
 }
@@ -904,7 +907,7 @@ void node_protocols_window_create(const node_id_t *node_id)
   list_item = g_list_find_custom(stats_info_windows,
                                  node_id, node_info_window_compare);
   if (!list_item) {
-    window = stats_info_create("node_id", g_memdup(node_id, sizeof(node_id_t)));
+    window = stats_info_create("node_id", g_memdup2(node_id, sizeof(node_id_t)));
     if (!window)
       return; /* creation failed */
 
@@ -967,7 +970,7 @@ void link_info_window_create(const link_id_t *link_id)
   list_item = g_list_find_custom(stats_info_windows,
                                  link_id, link_info_window_compare);
   if (!list_item) {
-    window = stats_info_create("link_id", g_memdup(link_id, sizeof(link_id_t)));
+    window = stats_info_create("link_id", g_memdup2(link_id, sizeof(link_id_t)));
     if (!window)
       return; /* creation failed */
   }
