@@ -101,7 +101,7 @@ int main(int argc, char *argv[])
     {"interface", 'i', POPT_ARG_STRING, &cl_interface, 0,
      N_("set interface to listen to"), N_("<interface name>")},
     {"final-export", 0, POPT_ARG_STRING, &export_file_final, 0,
-     N_("export to named file at end of replay"), N_("<file to export to>")},
+     N_("export to named file at end of replay and exit"), N_("<file to export to>")},
     {"signal-export", 0, POPT_ARG_STRING, &export_file_signal, 0,
      N_("export to named file on receiving USR1"), N_("<file to export to>")},
     {"position", 'P', POPT_ARG_STRING, &position_file_path, 0,
@@ -129,7 +129,8 @@ int main(int argc, char *argv[])
     {"relinquish-privileges", 'Z', POPT_ARG_STRING, &cl_privdrop_user, 0,
      N_("run as the given user"), N_("<username>")},
 
-    POPT_AUTOHELP {NULL, 0, 0, NULL, 0, NULL, NULL}
+    POPT_AUTOHELP
+    POPT_TABLEEND 
   };
 
 #ifdef ENABLE_NLS
@@ -426,21 +427,24 @@ static void set_debug_level(void)
   const gchar *env_debug;
   env_debug = g_getenv("APE_DEBUG");
 
+  printf("use APE_DEBUG and/or G_MESSAGES_DEBUG environment variables to control debug messages\n");
+
   appdata.debug_mask = (G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_INFO));
 
   if (env_debug) {
     if (!g_ascii_strcasecmp(env_debug, "INFO"))
       appdata.debug_mask = (G_LOG_LEVEL_MASK & ~G_LOG_LEVEL_DEBUG);
-    else if (!g_ascii_strcasecmp(env_debug, "DEBUG"))
+    else if (!g_ascii_strcasecmp(env_debug, "DEBUG")) {
       appdata.debug_mask = G_LOG_LEVEL_MASK;
+        // ugly workaround for changed g_log_default_handler behaviour
+        // unfortunately it can be controlled only by environment vars ...
+        // Silly change!
+        g_setenv("G_MESSAGES_DEBUG", "etherape", TRUE);
+    }
   }
   else
     appdata.debug_mask = (G_LOG_LEVEL_MASK & ~G_LOG_LEVEL_DEBUG);
 
-  // ugly workaround for changed g_log_default_handler behaviour
-  // unfortunately it can be controlled only by environment vars ...
-  // Silly change!
-  g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
 
   if (quiet)
     appdata.debug_mask = 0;
