@@ -57,6 +57,10 @@ void init_menus(void)
   if (err_str)
     g_string_free(err_str, TRUE);
 
+  /* if not already set, first interface is the default capture device */
+  if (!appdata.source.interface)
+    appdata.source.interface = g_strdup((gchar *)(interfaces->data));
+
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "interfaces_menu"));
 
   info_string = g_string_new(_("Available interfaces for capture:"));
@@ -392,32 +396,32 @@ static void set_ctrl_enablestate(guint32 flags)
 /* Sets up the GUI to reflect changes and calls start_capture() */
 void gui_start_capture(void)
 {
-  GtkWidget *widget;
-  gchar *errorbuf = NULL;
-  GString *status_string = NULL;
-
-  if (get_capture_status() == CAP_EOF)
+  if (get_capture_status() == CAP_EOF) {    
     if (!gui_stop_capture())
       return;
-
-
-
-  if (get_capture_status() == STOP) {
-    if ((errorbuf = start_capture()) != NULL) {
-      fatal_error_dialog(errorbuf);
-      return;
-    }
   }
-  else if (get_capture_status() == PLAY) {
+
+  gchar *errorbuf = NULL;
+  switch (get_capture_status())
+  {
+  case STOP:
+    errorbuf = start_capture();
+    break;
+  case PAUSE:
+    errorbuf = unpause_capture();
+    break;
+  case PLAY:
     g_warning(_("Status already PLAY at gui_start_capture"));
     return;
+  default:
+    g_warning(_("Unexpected status at gui_start_capture: %d"), get_capture_status());
+    return;
   }
-  else if (get_capture_status() == PAUSE) {
-    errorbuf = unpause_capture();
-    if (errorbuf) {
-      fatal_error_dialog(errorbuf);
-      return;
-    }
+
+  if (errorbuf)
+  {
+    fatal_error_dialog(errorbuf);
+    return;
   }
 
   in_start_capture = TRUE;
@@ -429,6 +433,7 @@ void gui_start_capture(void)
   set_widget_enabled_by_id("link_radio", has_linklevel());
 
   /* Set active mode in GUI */
+  GtkWidget *widget = NULL;
   switch (appdata.mode)
   {
       case LINK6:
@@ -450,7 +455,7 @@ void gui_start_capture(void)
   set_active_interface();
 
   /* Sets the statusbar */
-  status_string = g_string_new(_("Reading data from "));
+  GString *status_string = g_string_new(_("Reading data from "));
 
   if (appdata.source.type == ST_FILE && appdata.source.file)
     g_string_append(status_string, appdata.source.file);
@@ -481,19 +486,17 @@ void gui_start_capture(void)
   in_start_capture = FALSE;
 
   g_my_info(_("Diagram started"));
-}                               /* gui_start_capture */
+}
 
 void gui_pause_capture(void)
 {
-  gchar *err;
-
-  /*
+   /*
    * Make sure the data in the info windows is updated
    * so that it is consistent
    */
   update_info_windows(NULL);
 
-  err = pause_capture();
+  gchar *err = pause_capture();
   if (err) {
     g_error("Error pausing capture: %s", err);
     g_free(err);
@@ -530,7 +533,6 @@ void gui_eof_capture(void)
 gboolean gui_stop_capture(void)
 {
   GString *status_string = NULL;
-  gchar *err;
 
   stop_requested = FALSE;
   if (get_capture_status() == STOP)
@@ -548,12 +550,7 @@ gboolean gui_stop_capture(void)
     return FALSE;
   }
 
-  err = stop_capture();
-  if (err) {
-    g_error(_("Failed to stop capture: %s"), err);
-    g_free(err);
-    return FALSE;
-  }
+  stop_capture();
 
   set_ctrl_enablestate(EN_PLAY);
 
