@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <gtk/gtk.h>
@@ -28,6 +28,7 @@ void on_interface_radio_activate(gchar *gui_device);
 void on_mode_radio_activate(GtkRadioMenuItem *menuitem, gpointer user_data);
 void on_start_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_stop_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_refresh_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_pause_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_next_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data);
 
