@@ -31,6 +31,8 @@
 
 #include <glib.h>
 
+#include "stats/util.h"
+
 #define EPATH_ETHERS  "/etc/ethers"
 
 #define ETHBUFSIZE  4096
@@ -56,7 +58,7 @@ static void add_ether_ent(const struct ether_addr *addr, const char *hostname)
 {
   struct ether_ent *ent = g_malloc(sizeof(*ent) + strlen(hostname) + 1);
   ent->addr = *addr;
-  strcpy(ent->hostname, hostname);
+  safe_strncpy(ent->hostname, hostname, strlen(hostname) + 1);
   g_tree_insert(ethers, &ent->addr, ent);
 }
 
