@@ -145,11 +145,9 @@ static struct ipcache_item *ipcache_alloc_item(const address_t *ip)
 
 struct ipcache_item *ipcache_prepare_request(const address_t *ip)
 {
-  struct ipcache_item *item;
-
   g_assert(ip);
 
-  item = g_tree_lookup(ipcache_tree, ip);
+  struct ipcache_item *item = g_tree_lookup(ipcache_tree, ip);
   g_assert(!item);
   item = ipcache_alloc_item(ip);
 
