@@ -249,16 +249,18 @@ static void stop_threads()
 */
 static void sendrequest_inverse(address_t *ip)
 {
-  struct ipcache_item *rp = NULL;
-
   if (!ip)
     return;
 
   /* allocate a new request */
-  rp = ipcache_prepare_request(ip);
+  struct ipcache_item *rp = ipcache_prepare_request(ip);
 
   /* prepare resolve item */
   struct ipresolve_link *newitm = (struct ipresolve_link *)malloc(sizeof(struct ipresolve_link));
+  if (!newitm) {
+    g_my_debug("Resolver: failed to allocate memory for new request.");
+    return;
+  }
   newitm->itemToResolve = rp;
   newitm->next = NULL;
 
