@@ -128,7 +128,7 @@ void initialize_pref_controls(void)
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "size_mode"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(widget), pref.size_mode);
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "text_font"));
-  gtk_font_button_set_font_name(GTK_FONT_BUTTON(widget), pref.fontname);
+  gtk_font_chooser_set_font(GTK_FONT_CHOOSER(widget), pref.fontname);
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "text_color"));
   if (!gdk_rgba_parse(&color, pref.text_color))
     gdk_rgba_parse(&color, "#ffff00");
@@ -324,7 +324,7 @@ static void on_stack_level_changed(GtkWidget *combo, gpointer data)
 
 static void on_text_font_changed(GtkFontButton *wdg, gpointer data)
 {
-  const gchar *str = gtk_font_button_get_font_name(wdg);
+  const gchar *str = gtk_font_chooser_get_font(GTK_FONT_CHOOSER(wdg));
   if (str) {
     if (pref.fontname)
       g_free(pref.fontname);
@@ -496,8 +496,7 @@ void on_color_edit_dialog_show(GtkWidget *wdg, gpointer user_data)
   GtkTreePath *gpath = NULL;
   GtkTreeViewColumn *gcol = NULL;
   GtkTreeIter it;
-  GtkComboBox *cbox;
-  GtkColorChooser *chos;
+  gint isadd;
   EATreePos ep;
 
   if (!get_color_store(&ep))
@@ -507,15 +506,16 @@ void on_color_edit_dialog_show(GtkWidget *wdg, gpointer user_data)
   gtk_tree_view_get_cursor(ep.gv, &gpath, &gcol);
   if (gpath) {
     /* row selected */
-    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
+    if (!gtk_tree_model_get_iter
+            (GTK_TREE_MODEL(ep.gs), &it, gpath))
       return; /* path not found */
 
     gtk_tree_model_get(GTK_TREE_MODEL(ep.gs), &it, 1, &color, 2, &protocol_string, -1);
 
-    cbox = GTK_COMBO_BOX(gtk_builder_get_object(appdata.xml, "protocol_entry"));
+    GtkComboBox *cbox = GTK_COMBO_BOX(gtk_builder_get_object(appdata.xml, "protocol_entry"));
     cbox_add_select(cbox, protocol_string);
 
-    chos = GTK_COLOR_CHOOSER(gtk_builder_get_object(appdata.xml, "color_entry"));
+    GtkColorChooser *chos = GTK_COLOR_CHOOSER(gtk_builder_get_object(appdata.xml, "color_entry"));
     gtk_color_chooser_set_rgba(chos, color);
 
     g_free(protocol_string);
@@ -572,7 +572,7 @@ void on_color_remove_button_clicked(GtkButton *button, gpointer user_data)
 
 void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data)
 {
-  GtkWidget *colorsel, *colorseldiag;
+  GtkWidget *colorseldiag;
   GdkRGBA color;
   GtkTreePath *gpath = NULL;
   GtkTreeViewColumn *gcol = NULL;
@@ -594,6 +594,7 @@ void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data)
       if (!gtk_tree_model_get_iter
             (GTK_TREE_MODEL(ep.gs), &itsibling, gpath))
         return; /* path not found */
+
       gtk_list_store_insert_before(ep.gs, &it, &itsibling);
     }
     else
@@ -606,10 +607,7 @@ void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data)
   }
 
   /* get the selected color */
-  colorsel = gtk_color_selection_dialog_get_color_selection(
-    GTK_COLOR_SELECTION_DIALOG(colorseldiag));
-  gtk_color_selection_get_current_rgba(GTK_COLOR_SELECTION(colorsel),
-                                       &color);
+  gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(colorseldiag), &color);
 
   /* fill data */
   if (isadd)
@@ -664,6 +662,7 @@ void on_color_edit_ok_clicked(GtkButton *button, gpointer user_data)
       GtkTreeIter itsibling;
       if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &itsibling, gpath))
         return; /* path not found */
+
       gtk_list_store_insert_before(ep.gs, &it, &itsibling);
     }
     else
