@@ -17,7 +17,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <gtk/gtk.h>
@@ -33,12 +33,9 @@ void on_averaging_spin_adjustment_changed(GtkAdjustment *adj);
 void on_refresh_spin_adjustment_changed(GtkAdjustment *adj,
                                         GtkWidget *canvas);
 void on_node_radius_slider_adjustment_changed(GtkAdjustment *adj);
-void on_link_width_slider_adjustment_changed(GtkAdjustment *adj);
 void on_inner_ring_scale_slider_adjustment_changed(GtkAdjustment *adj);
-void on_node_to_spin_adjustment_changed(GtkAdjustment *adj);
 void on_gui_node_to_spin_adjustment_changed(GtkAdjustment *adj);
 void on_proto_node_to_spin_adjustment_changed(GtkAdjustment *adj);
-void on_link_to_spin_adjustment_changed(GtkAdjustment *adj);
 void on_gui_link_to_spin_adjustment_changed(GtkAdjustment *adj);
 void on_proto_link_to_spin_adjustment_changed(GtkAdjustment *adj);
 void on_proto_to_spin_adjustment_changed(GtkAdjustment *adj);
@@ -53,10 +50,9 @@ void on_background_image_path_selected(GtkFileChooserButton *filechooser, gpoint
 
 /* proto color tab callbacks */
 void on_color_add_button_clicked(GtkButton *button, gpointer user_data);
-void on_color_change_button_clicked(GtkButton *button, gpointer user_data);
+void on_color_edit_button_clicked(GtkButton *button, gpointer user_data);
 void on_color_remove_button_clicked(GtkButton *button, gpointer user_data);
+void on_color_edit_dialog_show(GtkWidget *wdg, gpointer user_data);
+void on_color_edit_ok_clicked(GtkButton *button, gpointer user_data);
 void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data);
-void on_protocol_edit_button_clicked(GtkButton *button, gpointer user_data);
-void on_protocol_edit_dialog_show(GtkWidget *wdg, gpointer user_data);
-void on_protocol_edit_ok_clicked(GtkButton *button, gpointer user_data);
 
