@@ -20,7 +20,7 @@
 #define PREFERENCES_H
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 #include "common.h"
 
@@ -35,12 +35,12 @@ struct pref_struct
   gboolean name_res;            /* Whether dns lookups are performed */
   gboolean diagram_only;        /* Do not use text on the diagram */
   gboolean group_unk;           /* Whether to display as one every unkown port protocol */
+  gboolean headless;            /* don't update display (TODO: implement a proper text mode) */
   gdouble node_radius_multiplier;       /* used to calculate the radius of the
                                          * displayed nodes. So that the user can
                                          * select with certain precision this
                                          * value, the GUI uses the log10 of the
                                          * multiplier */
-  gdouble link_node_ratio;      /* link width to node radius ratio */
   gdouble inner_ring_scale; /* scale of inner ring in proportion to outer ring */
   size_mode_t size_mode;        /* Default mode for node size and
                                  * link width calculation */
@@ -73,10 +73,6 @@ struct pref_struct
 
   /* After this time has passed with no traffic for a node, it
   * is deleted from memory */
-  gdouble node_timeout_time;
-
-  /* after this time has passed without traffic on a protocol, it's removed
-   * from the node protocol stats */
   gdouble proto_node_timeout_time;
 
   gchar * *colors;       /* list of colors to be used on the diagram. Format is
@@ -87,12 +83,7 @@ struct pref_struct
    * disappears from the diagram */
   gdouble gui_link_timeout_time;
 
-  /* After this time has passed with no traffic for a link, it
-  * is deleted from memory */
-  gdouble link_timeout_time;
-
-  /* after this time has passed without traffic on a protocol, it's removed
-   * from the link protocol stats */
+  /* after this time has passed without traffic, the link is removed */
   gdouble proto_link_timeout_time;
 
   guint32 refresh_period;       /* Time between diagram refreshes */
@@ -106,7 +97,7 @@ extern struct pref_struct pref;
 extern GList *centered_node_speclist;
 
 /* preferences methods */
-void load_config(struct pref_struct *p);
+gboolean load_config(struct pref_struct *p);
 void save_config(const struct pref_struct *p);
 void free_config(struct pref_struct *t);
 void copy_config(struct pref_struct *tgt, const struct pref_struct *src);
