@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../../config.h"
+#include "config.h"
 #endif
 
 #include "appdata.h"
@@ -69,7 +69,7 @@ enum rpc_program
   KERBPROG_PROGRAM = 100078
 };
 
-/* internal types */
+/* internal struct - decode helper */
 typedef struct
 {
   const guint8 *original_packet; /* original start of packet */
@@ -78,8 +78,10 @@ typedef struct
   const guint8 *cur_packet; /* pointer to current level start of packet */
   guint cur_len;        /* current level remaining length */
 
-  packet_protos_t *pr; /* detected protocol stack */
-  guint cur_level;      /* current protocol depth on stack */
+  /* decode outputs */ 
+
+  packet_protos_t *pr_stack; /* detected protocol stack */
+  guint pr_cur_level;      /* current protocol depth on stack */
 
   /* node ids */
   node_id_t dst_node_id;
@@ -93,18 +95,18 @@ typedef struct
 } decode_proto_t;
 
 /* extracts the protocol stack from packet */
-static void get_packet_prot(decode_proto_t *dp);
+static void decode_protocol_stack(decode_proto_t *dp, packet_protos_t *protostack_tofill,
+                                  const guint8 *pkt, guint cap_len);
 
 /* sets protoname at current level, and passes at next level */
-static void decode_proto_add(decode_proto_t *dp, const gchar *fmt, ...);
+static void decode_proto_add(decode_proto_t *dp, const gchar *proto);
+static void decode_proto_add_fmt(decode_proto_t *dp, const gchar *fmt, ...);
 
 /* advances current packet start to prepare for next protocol */
 static void add_offset(decode_proto_t *dp, guint offset);
 
-static void add_node_packet(const guint8 *packet,
-                            guint raw_size,
-                            packet_info_t *packet_info,
-                            const node_id_t *node_id,
+static void add_node_packet(node_t *node,
+                            packet_info_t *packet,
                             packet_direction direction);
 
 /* decoder func proto */
@@ -207,31 +209,23 @@ gboolean has_linklevel(void)
 /* ------------------------------------------------------------
  * Implementation
  * ------------------------------------------------------------*/
-/* starts a new decode, allocating a new packet_protos_t */
-void decode_proto_start(decode_proto_t *dp, packet_protos_t *protos,
-                        const guint8 *pkt, guint caplen)
+static void decode_proto_add(decode_proto_t *dp, const gchar *proto)
 {
-  dp->original_packet = pkt;
-  dp->original_len = caplen;
-  dp->cur_packet = pkt;
-  dp->cur_len = caplen;
-  dp->pr = protos;
-  dp->cur_level = 1; /* level zero is topmost protocol, will be filled later */
-  node_id_clear(&dp->dst_node_id);
-  node_id_clear(&dp->src_node_id);
-  address_clear(&dp->global_src_address);
-  address_clear(&dp->global_dst_address);
-  dp->global_src_port = 0;
-  dp->global_dst_port = 0;
+  if (dp->pr_cur_level <= STACK_SIZE) {
+    dp->pr_stack->protonames[dp->pr_cur_level] = g_strdup(proto);
+    dp->pr_cur_level++;
+  }
+  else
+    g_warning("protocol \"%.10s\" too deeply nested, ignored", proto ? proto : "");
 }
-void decode_proto_add(decode_proto_t *dp, const gchar *fmt, ...)
+static void decode_proto_add_fmt(decode_proto_t *dp, const gchar *fmt, ...)
 {
   va_list ap;
-  if (dp->cur_level <= STACK_SIZE) {
+  if (dp->pr_cur_level <= STACK_SIZE) {
     va_start(ap, fmt);
-    dp->pr->protonames[dp->cur_level] = g_strdup_vprintf(fmt, ap);
+    dp->pr_stack->protonames[dp->pr_cur_level] = g_strdup_vprintf(fmt, ap);
     va_end(ap);
-    dp->cur_level++;
+    dp->pr_cur_level++;
   }
   else
     g_warning("protocol \"%.10s\" too deeply nested, ignored", fmt ? fmt : "");
@@ -256,6 +250,8 @@ void packet_acquired(guint8 *cap_bytes, guint cap_size, guint orig_size)
   packet_info_t *packet;
   link_id_t link_id;
   decode_proto_t decp;
+  node_t *src_node;
+  node_t *dst_node;
 
   if (!lkentry || !lkentry->fun) {
     g_error(_("Data link entry not initialized"));
@@ -270,20 +266,37 @@ void packet_acquired(guint8 *cap_bytes, guint cap_size, guint orig_size)
   packet->ref_count = 0;
   memset(&packet->prot_desc, 0, sizeof(packet->prot_desc));
 
-  decode_proto_start(&decp, &packet->prot_desc, cap_bytes, cap_size);
-
-  /* Get the protocol tree */
-  get_packet_prot(&decp);
+  /* decode the protocol tree */
+  decode_protocol_stack(&decp, &packet->prot_desc, cap_bytes, cap_size);
 
   appdata.n_packets++;
   appdata.total_mem_packets++;
 
-  /* Add this packet information to the src and dst nodes. If they
-   * don't exist, create them */
-  add_node_packet(cap_bytes, cap_size, packet, &decp.src_node_id, OUTBOUND);
-  add_node_packet(cap_bytes, cap_size, packet, &decp.dst_node_id, INBOUND);
+  src_node = nodes_catalog_find(&decp.src_node_id);
+  if (src_node == NULL) {
+    /* creates the new node, adding it to the catalog */
+    src_node = nodes_catalog_new(&decp.src_node_id);
+    g_assert(src_node);
+  }
+  dst_node = nodes_catalog_find(&decp.dst_node_id);
+  if (dst_node == NULL) {
+    /* creates the new node, adding it to the catalog */
+    dst_node = nodes_catalog_new(&decp.dst_node_id);
+    g_assert(dst_node);
+  }
 
-  /* And now we update link traffic information for this packet */
+  /* Add this packet information to the src and dst nodes */
+  add_node_packet(src_node, packet, OUTBOUND);
+  add_node_packet(dst_node, packet, INBOUND);
+
+  /* Update names list for these nodes */
+  get_packet_names(&src_node->node_stats.stats_protos, &dst_node->node_stats.stats_protos, 
+                   cap_bytes, cap_size, &packet->prot_desc, lkentry->dlt_linktype);
+
+  /* And now we update link traffic information for this packet 
+     packets originating from lesser addresses are arbitrarily marked outbound
+     link ids are pairs of node addresses, with the lesser one first
+  */
   if (node_id_compare(&decp.src_node_id, &decp.dst_node_id) < 1) {
     /* src id <= dst id, direct packet */
     link_id.src = decp.src_node_id;
@@ -291,7 +304,7 @@ void packet_acquired(guint8 *cap_bytes, guint cap_size, guint orig_size)
     links_catalog_add_packet(&link_id, packet, OUTBOUND);
   }
   else {
-    /* src id <= dst id, inverse packet */
+    /* src id > dst id, inverse packet */
     link_id.src = decp.dst_node_id;
     link_id.dst = decp.src_node_id;
     links_catalog_add_packet(&link_id, packet, INBOUND);
@@ -305,20 +318,10 @@ void packet_acquired(guint8 *cap_bytes, guint cap_size, guint orig_size)
 /* We update node information for each new packet that arrives in the
  * network. If the node the packet refers to is unknown, we
  * create it. */
-static void add_node_packet(const guint8 *raw_packet,
-                            guint raw_size,
+static void add_node_packet(node_t *node,
                             packet_info_t *packet,
-                            const node_id_t *node_id,
                             packet_direction direction)
 {
-  node_t *node;
-
-  node = nodes_catalog_find(node_id);
-  if (node == NULL) {
-    /* creates the new node, adding it to the catalog */
-    node = nodes_catalog_new(node_id);
-    g_assert(node);
-  }
 
   traffic_stats_add_packet(&node->node_stats, packet, direction);
 
@@ -327,27 +330,40 @@ static void add_node_packet(const guint8 *raw_packet,
    * node is active again */
   if (node->node_stats.pkt_list.length == 1)
     new_nodes_add(node);
+} 
 
-  /* Update names list for this node */
-  get_packet_names(&node->node_stats.stats_protos, raw_packet, raw_size,
-                   &packet->prot_desc, direction, lkentry->dlt_linktype);
-}                               /* add_node_packet */
-
-static void get_packet_prot(decode_proto_t *dp)
+static void decode_protocol_stack(decode_proto_t *dp, packet_protos_t *protostack_tofill,
+                                  const guint8 *pkt, guint cap_len)
 {
   guint i;
 
+  /* initialize data helper */
+  dp->original_packet = pkt;
+  dp->original_len = cap_len;
+  dp->cur_packet = pkt;
+  dp->cur_len = cap_len;
+  dp->pr_stack = protostack_tofill;
+  dp->pr_cur_level = 1; /* level zero is topmost protocol, will be filled later */
+  node_id_clear(&dp->dst_node_id);
+  node_id_clear(&dp->src_node_id);
+  address_clear(&dp->global_src_address);
+  address_clear(&dp->global_dst_address);
+  dp->global_src_port = 0;
+  dp->global_dst_port = 0;
+
+
+  /* call current link decode function */
   g_assert(lkentry && lkentry->fun);
   lkentry->fun(dp);
 
   /* first position is top proto */
   for (i = STACK_SIZE; i > 0; --i) {
-    if (dp->pr->protonames[i]) {
-      dp->pr->protonames[0] = g_strdup(dp->pr->protonames[i]);
+    if (dp->pr_stack->protonames[i]) {
+      dp->pr_stack->protonames[0] = g_strdup(dp->pr_stack->protonames[i]);
       break;
     }
   }
-}                               /* get_packet_prot */
+}
 
 /* ------------------------------------------------------------
  * Private functions
@@ -427,10 +443,10 @@ static void get_eth_type(decode_proto_t *dp)
 
   /* node ids */
   dp->dst_node_id.node_type = LINK6;
-  g_memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 0,
+  memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 0,
             sizeof(dp->dst_node_id.addr.eth));
   dp->src_node_id.node_type = LINK6;
-  g_memmove(dp->src_node_id.addr.eth, dp->cur_packet + 6,
+  memmove(dp->src_node_id.addr.eth, dp->cur_packet + 6,
             sizeof(dp->src_node_id.addr.eth));
 
   add_offset(dp, size_offset + 2);
@@ -477,10 +493,10 @@ static void get_fddi_type(decode_proto_t *dp)
 
   /* node ids */
   dp->dst_node_id.node_type = LINK6;
-  g_memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 1,
+  memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 1,
             sizeof(dp->dst_node_id.addr.eth));
   dp->src_node_id.node_type = LINK6;
-  g_memmove(dp->src_node_id.addr.eth, dp->cur_packet + 7,
+  memmove(dp->src_node_id.addr.eth, dp->cur_packet + 7,
             sizeof(dp->src_node_id.addr.eth));
 
   /* Ok, this is only temporary while I truly dissect LLC
@@ -503,10 +519,10 @@ static void get_ieee802_5_type(decode_proto_t *dp)
 
   /* node ids */
   dp->dst_node_id.node_type = LINK6;
-  g_memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 2,
+  memmove(dp->dst_node_id.addr.eth, dp->cur_packet + 2,
             sizeof(dp->dst_node_id.addr.eth));
   dp->src_node_id.node_type = LINK6;
-  g_memmove(dp->src_node_id.addr.eth, dp->cur_packet + 8,
+  memmove(dp->src_node_id.addr.eth, dp->cur_packet + 8,
             sizeof(dp->src_node_id.addr.eth));
 
   if (dp->cur_len < 22)
@@ -728,7 +744,7 @@ static void get_wlan(decode_proto_t *dp)
     return;
   }
   dp->dst_node_id.node_type = LINK6;
-  g_memmove(dp->dst_node_id.addr.eth, dp->cur_packet + dstofs,
+  memmove(dp->dst_node_id.addr.eth, dp->cur_packet + dstofs,
             sizeof(dp->dst_node_id.addr.eth));
 
   if (type == 1) {
@@ -742,7 +758,7 @@ static void get_wlan(decode_proto_t *dp)
         return;
       }
       dp->src_node_id.node_type = LINK6;
-      g_memmove(dp->src_node_id.addr.eth, dp->cur_packet + srcofs,
+      memmove(dp->src_node_id.addr.eth, dp->cur_packet + srcofs,
                 sizeof(dp->src_node_id.addr.eth));
       add_offset(dp, 16);
     }
@@ -758,7 +774,7 @@ static void get_wlan(decode_proto_t *dp)
     return;
   }
   dp->src_node_id.node_type = LINK6;
-  g_memmove(dp->src_node_id.addr.eth, dp->cur_packet + srcofs,
+  memmove(dp->src_node_id.addr.eth, dp->cur_packet + srcofs,
             sizeof(dp->src_node_id.addr.eth));
 
   if (fromtods != 3)
@@ -942,7 +958,7 @@ static void get_llc(decode_proto_t *dp)
 
 static void get_ip(decode_proto_t *dp)
 {
-  guint16 fragment_offset;
+  guint16 fragment_offset = 0;
   iptype_t ip_type;
   int ip_version, ip_hl;
 
@@ -953,6 +969,7 @@ static void get_ip(decode_proto_t *dp)
   switch (ip_version)
   {
       case 4:
+        // ipv4
         ip_hl = (dp->cur_packet[0] & 15) << 2;
         if (ip_hl < 20)
           return;
@@ -967,34 +984,34 @@ static void get_ip(decode_proto_t *dp)
           dp->dst_node_id.node_type = IP;
           address_clear(&dp->dst_node_id.addr.ip);
           dp->dst_node_id.addr.ip.type = AF_INET;
-          g_memmove(dp->dst_node_id.addr.ip.addr_v4, dp->cur_packet + 16,
+          memmove(dp->dst_node_id.addr.ip.addr_v4, dp->cur_packet + 16,
                     sizeof(dp->dst_node_id.addr.ip.addr_v4));
           dp->src_node_id.node_type = IP;
           address_clear(&dp->src_node_id.addr.ip);
           dp->src_node_id.addr.ip.type = AF_INET;
-          g_memmove(dp->src_node_id.addr.ip.addr_v4, dp->cur_packet + 12,
+          memmove(dp->src_node_id.addr.ip.addr_v4, dp->cur_packet + 12,
                     sizeof(dp->src_node_id.addr.ip.addr_v4));
         }
 
         add_offset(dp, ip_hl);
         break;
       case 6:
+        // ipv6
         if (dp->cur_len < 40)
           return;
         decode_proto_add(dp, "IPV6");
 
         ip_type = dp->cur_packet[6];
-        fragment_offset = 0;
 
         if (appdata.mode !=  LINK6) {
           /* we want higher level node ids */
           dp->dst_node_id.node_type = IP;
           dp->dst_node_id.addr.ip.type = AF_INET6;
-          g_memmove(dp->dst_node_id.addr.ip.addr_v6, dp->cur_packet + 24,
+          memmove(dp->dst_node_id.addr.ip.addr_v6, dp->cur_packet + 24,
                     sizeof(dp->dst_node_id.addr.ip.addr_v6));
           dp->src_node_id.node_type = IP;
           dp->src_node_id.addr.ip.type = AF_INET6;
-          g_memmove(dp->src_node_id.addr.ip.addr_v6, dp->cur_packet + 8,
+          memmove(dp->src_node_id.addr.ip.addr_v6, dp->cur_packet + 8,
                     sizeof(dp->src_node_id.addr.ip.addr_v6));
         }
 
@@ -1034,14 +1051,18 @@ static void get_ip(decode_proto_t *dp)
       case IP_PROTO_IPIP:
         decode_proto_add(dp, "IPIP");
         break;
-#if 0                           /* TODO How come IPIP and IPV4 share the same number? */
-      case IP_PROTO_IPV4:
-        decode_proto_add(dp, "IPV4");
+      case IP_PROTO_ST:
+        decode_proto_add(dp, "ST");
+        break;
+      case IP_PROTO_CBT:
+        decode_proto_add(dp, "CBT");
         break;
-#endif
       case IP_PROTO_EGP:
         decode_proto_add(dp, "EGP");
         break;
+      case IP_PROTO_IGP:
+        decode_proto_add(dp, "IGP");
+        break;
       case IP_PROTO_PUP:
         decode_proto_add(dp, "PUP");
         break;
@@ -1073,7 +1094,7 @@ static void get_ip(decode_proto_t *dp)
         decode_proto_add(dp, "AH");
         break;
       case IP_PROTO_ICMPV6:
-        decode_proto_add(dp, "ICPMPV6");
+        decode_proto_add(dp, "ICMPV6");
         break;
       case IP_PROTO_NONE:
         decode_proto_add(dp, "NONE");
@@ -1230,13 +1251,13 @@ static void get_tcp(decode_proto_t *dp)
      * to already have an IP node id */
     g_assert(dp->dst_node_id.node_type == IP);
     dp->dst_node_id.node_type = TCP;
-    address_copy(&dp->dst_node_id.addr.tcp4.host, &dp->global_dst_address);
-    dp->dst_node_id.addr.tcp4.port = dp->global_dst_port;
+    address_copy(&dp->dst_node_id.addr.tcp.host, &dp->global_dst_address);
+    dp->dst_node_id.addr.tcp.port = dp->global_dst_port;
 
     g_assert(dp->src_node_id.node_type == IP);
     dp->src_node_id.node_type = TCP;
-    address_copy(&dp->src_node_id.addr.tcp4.host, &dp->global_src_address);
-    dp->src_node_id.addr.tcp4.port = dp->global_src_port;
+    address_copy(&dp->src_node_id.addr.tcp.host, &dp->global_src_address);
+    dp->src_node_id.addr.tcp.port = dp->global_src_port;
   }
 
   th_off_x2 = *(guint8 *)(dp->cur_packet + 12);
@@ -1275,9 +1296,9 @@ static void get_tcp(decode_proto_t *dp)
       decode_proto_add(dp, "TCP-UNKNOWN");
     else {
       if (chosen_port == src_port)
-        decode_proto_add(dp, "TCP:%d-%d", chosen_port, dst_port);
+        decode_proto_add_fmt(dp, "TCP:%d-%d", chosen_port, dst_port);
       else
-        decode_proto_add(dp, "TCP:%d-%d", chosen_port, src_port);
+        decode_proto_add_fmt(dp, "TCP:%d-%d", chosen_port, src_port);
     }
     return;
   }
@@ -1298,7 +1319,7 @@ static void get_tcp(decode_proto_t *dp)
   else
     chosen_service = dst_service;
 
-  decode_proto_add(dp, "%s", chosen_service->name);
+  decode_proto_add(dp, chosen_service->name);
 }                               /* get_tcp */
 
 static void get_udp(decode_proto_t *dp)
@@ -1317,13 +1338,13 @@ static void get_udp(decode_proto_t *dp)
      * to already have an IP node id */
     g_assert(dp->dst_node_id.node_type == IP);
     dp->dst_node_id.node_type = TCP;
-    address_copy(&dp->dst_node_id.addr.tcp4.host, &dp->global_dst_address);
-    dp->dst_node_id.addr.tcp4.port = dp->global_dst_port;
+    address_copy(&dp->dst_node_id.addr.tcp.host, &dp->global_dst_address);
+    dp->dst_node_id.addr.tcp.port = dp->global_dst_port;
 
     g_assert(dp->src_node_id.node_type == IP);
     dp->src_node_id.node_type = TCP;
-    address_copy(&dp->src_node_id.addr.tcp4.host, &dp->global_src_address);
-    dp->src_node_id.addr.tcp4.port = dp->global_src_port;
+    address_copy(&dp->src_node_id.addr.tcp.host, &dp->global_src_address);
+    dp->src_node_id.addr.tcp.port = dp->global_src_port;
   }
 
   add_offset(dp, 8);
@@ -1348,9 +1369,9 @@ static void get_udp(decode_proto_t *dp)
       decode_proto_add(dp, "UDP-UNKNOWN");
     else {
       if (chosen_port == src_port)
-        decode_proto_add(dp, "UDP:%d-%d", chosen_port, dst_port);
+        decode_proto_add_fmt(dp, "UDP:%d-%d", chosen_port, dst_port);
       else
-        decode_proto_add(dp, "UDP:%d-%d", chosen_port, src_port);
+        decode_proto_add_fmt(dp, "UDP:%d-%d", chosen_port, src_port);
     }
     return;
   }
@@ -1370,7 +1391,7 @@ static void get_udp(decode_proto_t *dp)
     chosen_service = src_service;
   else
     chosen_service = dst_service;
-  decode_proto_add(dp, "%s", chosen_service->name);
+  decode_proto_add(dp, chosen_service->name);
 }
 
 static gboolean get_rpc(decode_proto_t *dp, gboolean is_udp)
