@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../../config.h"
+#include "config.h"
 #endif
 
 #include <fnmatch.h>
@@ -76,15 +76,16 @@ static gint nodes_num = 0;      /* nodes counter */
  * node_t implementation
  *
  **************************************************************************/
-static void node_name_update(node_t *node);
+static void node_protocol_sort(node_t *node);
 static void set_node_name(node_t *node, const name_decode_t *sequence);
+static gchar *node_xml(node_t *node);
+
 
 /* Allocates a new node structure */
 node_t *node_create(const node_id_t *node_id)
 {
   node_t *node;
   gchar *name;
-  guint i;
 
   g_assert(node_id);
 
@@ -98,9 +99,6 @@ node_t *node_create(const node_id_t *node_id)
   node->numeric_name = g_string_new(name);
   g_free(name);
 
-  for (i = 0; i <= STACK_SIZE; ++i)
-    node->main_prot[i] = NULL;
-
   traffic_stats_init(&node->node_stats);
 
   ++nodes_num;
@@ -117,8 +115,6 @@ node_t *node_create(const node_id_t *node_id)
 /* destroys a node */
 void node_delete(node_t *node)
 {
-  guint i;
-
   if (!node)
     return; /* nothing to do */
 
@@ -129,12 +125,6 @@ void node_delete(node_t *node)
     g_string_free(node->numeric_name, TRUE);
   node->numeric_name = NULL;
 
-  for (i = 0; i <= STACK_SIZE; ++i)
-    if (node->main_prot[i]) {
-      g_free(node->main_prot[i]);
-      node->main_prot[i] = NULL;
-    }
-
   traffic_stats_reset(&node->node_stats);
 
   if (INFO_ENABLED) {
@@ -158,6 +148,7 @@ gchar *node_dump(const node_t *node)
   gchar *msg_id;
   gchar *msg_stats;
   gchar *msg_mprot;
+  const gchar *main_prot;
   guint i;
 
   if (!node)
@@ -166,15 +157,15 @@ gchar *node_dump(const node_t *node)
   msg_id = node_id_dump(&node->node_id);
   msg_stats = traffic_stats_dump(&node->node_stats);
 
+  main_prot = traffic_stats_most_used_proto(&node->node_stats, 0);
   msg_mprot = g_strdup_printf("top: [%s], stack:",
-                              (node->main_prot[0]) ?
-                              node->main_prot[0] : "-none-");
+                              (main_prot) ? main_prot : "-none-");
 
   for (i = 1; i <= STACK_SIZE; i++) {
     gchar *tmp = msg_mprot;
+    main_prot = traffic_stats_most_used_proto(&node->node_stats, i);
     msg_mprot = g_strdup_printf("%s %d:>%s<", msg_mprot, i,
-                                (node->main_prot[i]) ?
-                                node->main_prot[i] : "-none-");
+                                (main_prot) ? main_prot : "-none-");
     g_free(tmp);
   }
 
@@ -189,11 +180,8 @@ gchar *node_dump(const node_t *node)
   return msg;
 }
 
-/* returns a newly allocated string with an xml dump of node
- * N.B.
- * ignores main_prot array (protocol names), because is already dumped
- * with protostack stats */
-gchar *node_xml(const node_t *node)
+/* returns a newly allocated string with an xml dump of node */
+gchar *node_xml(node_t *node)
 {
   gchar *msg;
   gchar *msg_id;
@@ -204,10 +192,13 @@ gchar *node_xml(const node_t *node)
   if (!node)
     return xmltag("node", "");
 
+  /* first update protocol sorting to guarantee up-to-date ordering */
+  node_protocol_sort(node);
+
   msg_id = node_id_xml(&node->node_id);
   msg_stats = traffic_stats_xml(&node->node_stats);
-  msg_resolved = xmltag("resolved_name", "%s", node->name->str);
-  msg_numeric = xmltag("numeric_name", "%s", node->numeric_name->str);
+  msg_resolved = xmltag_escaped("resolved_name", "%s", node->name->str);
+  msg_numeric = xmltag_escaped("numeric_name", "%s", node->numeric_name->str);
 
   msg = xmltag("node", "\n<name>\n%s%s%s</name>\n%s",
                msg_id, msg_resolved, msg_numeric,
@@ -231,22 +222,15 @@ static gboolean node_update(node_id_t *node_id, node_t *node, gpointer delete_li
 
   if (traffic_stats_update(&node->node_stats, pref.averaging_time,
                            pref.proto_node_timeout_time)) {
-    /* packet(s) active, update the most used protocols for this link */
-    guint i = STACK_SIZE;
-    while (i + 1) {
-      if (node->main_prot[i])
-        g_free(node->main_prot[i]);
-      node->main_prot[i] = protocol_stack_sort_most_used(&node->node_stats.stats_protos, i);
-      i--;
-    }
-    node_name_update(node);
+    /* packet(s) active, update name and protocols for this link */
+    node_protocol_sort(node);
   }
   else {
     /* no packets remaining on node - if node expiration active, see if the
      * node is expired */
-    if (pref.node_timeout_time) {
+    if (pref.proto_node_timeout_time) {
       diffms = subtract_times_ms(&appdata.now, &node->node_stats.stats.last_time);
-      if (diffms >= pref.node_timeout_time) {
+      if (diffms >= pref.proto_node_timeout_time) {
         /* node expired, remove */
         GList * *delete_list = (GList * *)delete_list_ptr;
 
@@ -271,11 +255,14 @@ static gboolean node_update(node_id_t *node_id, node_t *node, gpointer delete_li
 
 /* Sets the node->name and node->numeric_name to the most used of
  * the default name for the current mode */
-static void node_name_update(node_t *node)
+static void node_protocol_sort(node_t *node)
 {
   GList *protocol_item;
   protocol_t *protocol;
-  guint i = STACK_SIZE;
+  guint i;
+
+  /* sort the protocol stack by most used */
+  protocol_stack_sort_most_used(&node->node_stats.stats_protos);
 
   /* for each level and each protocol at that level, sort names by traffic,
    * placing the busiest at front */
@@ -376,7 +363,7 @@ static void set_node_name(node_t *node, const name_decode_t *sequence)
     }
   }
   g_my_debug("set_node_name END --");
-}                               /* set_node_name */
+}
 
 
 /***************************************************************************
@@ -585,7 +572,7 @@ static gboolean node_xml_tvs(gpointer key, gpointer value, gpointer data)
   gchar *msg_node;
   gchar *tmp;
   gchar * *msg = (gchar * *)data;
-  const node_t *node = (const node_t *)value;
+  node_t *node = (node_t *)value;
 
   msg_node = node_xml(node);
   tmp = *msg;
@@ -724,7 +711,7 @@ static int nodeset_match(const struct nodeset_spec *spec, const node_t *node)
   if (node->node_id.node_type == IP)
     nodeaddr = &node->node_id.addr.ip;
   else if (node->node_id.node_type == TCP)
-    nodeaddr = &node->node_id.addr.tcp4.host;
+    nodeaddr = &node->node_id.addr.tcp.host;
   else
     return 0;
 
