@@ -47,6 +47,8 @@ typedef struct
 /* protocol stack methods */
 void protocol_stack_open(protostack_t *pstk);
 void protocol_stack_reset(protostack_t *pstk);
+/* sums another stack */
+void protocol_stack_sum(protostack_t *pstk, const protostack_t *tosum);
 /* adds packet data to the stack */
 void protocol_stack_add_pkt(protostack_t *pstk, const packet_info_t *packet);
 /* subtracts packet data from stack */
@@ -57,21 +59,25 @@ void protocol_stack_avg(protostack_t *pstk, gdouble avg_usecs);
 void protocol_stack_purge_expired(protostack_t *pstk, double expire_time);
 /* finds named protocol in the requested level of protostack*/
 const protocol_t *protocol_stack_find(const protostack_t *pstk, size_t level, const gchar *protoname);
-/* sorts on the most used protocol in the requested level and returns it */
-gchar *protocol_stack_sort_most_used(protostack_t *pstk, size_t level);
+/* sorts the stack on the most used protocol */
+void protocol_stack_sort_most_used(protostack_t *pstk);
+/* returns from the stack the name of most used protocol at the specified level, if present (NULL otherwise) */
+const gchar *protocol_stack_most_used(const protostack_t *pstk, size_t level);
 /* returns a newly allocated string with a dump of pstk */
 gchar *protocol_stack_dump(const protostack_t *pstk);
 /* returns a newly allocated string with am xml dump of pstk */
-gchar *protocol_stack_xml(const protostack_t *pstk);
+gchar *protocol_stack_xml(const protostack_t *pstk, const gchar *tag);
 
-/* protocol summary method */
+/* protocol summary methods */
 void protocol_summary_open(void); /* initializes the summary */
 void protocol_summary_close(void); /* frees summary, releasing resources */
 void protocol_summary_add_packet(packet_info_t *packet); /* adds a new packet to summary */
 void protocol_summary_update_all(void); /* update stats on protocol summary */
 long protocol_summary_size(void); /* total number of protos */
 void protocol_summary_foreach(size_t level, GFunc func, gpointer data); /* calls func for every proto at level */
+gchar *protocol_summary_xml(void);    /* returns a newly allocated string with an xml dump of protocol summary */
 const protocol_t *protocol_summary_find(size_t level, const gchar *protoname); /* finds named protocol */
 const protostack_t *protocol_summary_stack(void); /* access directly the stack (only for proto windows) */
+void protocol_summary_recalc_fromlinks(void);  /* recalcs procotol summary stats from link statistics */
 
 #endif
